from math import cos, sin

from fyui.game.sprite import Sprite

class Bullet(Sprite):
    def __init__(self, pos, bullet_type, angle=0., speed=0.):
        Sprite.__init__(self, bullet_type.texcoords,
                        blend_mode=bullet_type.blend_mode,
                        automatic_rotation=bullet_type.automatic_rotation,
                        hitbox=bullet_type.hitbox)

        self.bullet_type = bullet_type

        self.x, self.y = pos

        self.accel = 0.
        self.rotation_speed = 0.

        self.speed = speed
        self.angle = angle


    def set_disappearing(self, nb_frames=20, scale_modifier=0.8):
        self.frame = 0
        speed, angle = self.speed * 2. / 3., self.angle
        dx, dy = cos(angle) * speed, sin(angle) * speed
        def new_update():
            self.x, self.y = self.x + dx, self.y + dy
            self.frame += 1
            if self.frame < nb_frames:
                self.scale_factor += scale_modifier / nb_frames
                self.alpha -= 255 // nb_frames
                return True
            return False
        self.update = new_update


    def update(self):
        self.speed += self.accel
        self.angle += self.rotation_speed

        self.x += cos(self.angle) * self.speed
        self.y += sin(self.angle) * self.speed

