import weakref
from random import Random

from math import pi

from fyui.game.bullettype import default_bullet_types

from fyui.game.gamestate import GameState
from fyui.game.player import Player
from fyui.game.boss import Boss
from fyui.game.bullet import Bullet


class Game(object):
    class State:
        PLAYING, WON, LOST = range(3)

    def __init__(self, width, height, bullet_types=default_bullet_types, seed=0, seed2=0, survival_mode=False):
        self.survival_mode = survival_mode
        self.seed = seed
        self.seed2 = seed2
        self.score = 0
        self.nb_takedowns = 0
        self.state = GameState.PLAYING
        self.width, self.height = width, height
        self.bullet_types = bullet_types
        self.drones = [None, None]
        self.bullets = []
        self.player_bullets = []
        self.pending_deletion = []
        self.prng = Random(self.seed)
        self.prng2 = Random(self.seed2)

        self.player = Player(weakref.proxy(self))
        self.boss = Boss(weakref.proxy(self))


    def clear_bullets(self):
        self.pending_deletion.extend(self.bullets)

        for bullet in self.bullets:
            bullet.set_disappearing()

        self.bullets = []


    def update(self, keystate):
        if self.state != GameState.PLAYING:
            return

        # Update player, boss and drones
        self.player.update(keystate)
        self.boss.update()
        for i, drone in enumerate(self.drones):
            if drone and not drone.update():
                self.drones[i] = None

        # Update bullets
        for bullet in tuple(self.bullets):
            bullet.update()
        for bullet in tuple(self.player_bullets):
            bullet.update()

        # Check for collisions
        # 1. Check for bullets colliding with the player
        for bullet in tuple(self.bullets):
            if bullet.collide(self.player):
                self.player.on_hit()

        if self.player.lives < 0:
            self.state = GameState.LOST

        # 2. Check for bullets colliding with the boss
        for bullet in tuple(self.player_bullets):
            for drone in self.drones:
                if drone is not None and bullet.collide(drone):
                    drone.on_hit()
                    self.score += 10
                    self.player_bullets.remove(bullet)
                    bullet.set_disappearing()
                    self.pending_deletion.append(bullet)
        for bullet in tuple(self.player_bullets):
            if bullet.collide(self.boss):
                self.score += 10 + self.player.bombs
                self.boss.on_hit()

                self.player_bullets.remove(bullet)
                bullet.set_disappearing()
                self.pending_deletion.append(bullet)

        # Remove "dead" bullets
        for bullet in tuple(self.pending_deletion):
            if not bullet.update():
                self.pending_deletion.remove(bullet)

        # Clean out-of-screen bullets
        width, height = self.width, self.height
        self.bullets = [bullet for bullet in self.bullets if bullet.is_visible(width, height)]
        self.player_bullets = [bullet for bullet in self.player_bullets if bullet.is_visible(width, height)]

