from fyui.game.sprite import Sprite
from fyui.game.bullet import Bullet

from math import cos, sin, sqrt, pi

S2 = sqrt(2) / 2

class Player(Sprite):
    def __init__(self, game):
        Sprite.__init__(self, (0., 0., 36., 36.), hitbox=(-2, -2, 2, 2))
        self.game = game

        self.lives = 2
        self.bombs = 0
        self.slow_mode = False
        self.x, self.y = 200.0, 100.0

        self.hit = False

        self.frame = 0
        self.last_shot = -10
        self.last_bomb = -120
        self.next_bomb_completion = 0.

        self.effects = []
        self.bomb_particles = []

        self.update = self.main_update


    def on_hit(self):
        if self.hit:
            return

        self.frame = 0

        self.update = self.update2


    def launch_bomb(self):
        self.bombs -= 1
        self.last_bomb = self.frame
        self.bomb_particles = [Sprite((0, 0, 16, 16), blend_mode=1, pos=(self.x, self.y))
                                for i in xrange(7)]


    def update_dead(self, keystate):
        self.game.clear_bullets()
        self.effects[0].alpha -= 1
        self.effects[0].scale_factor += .4
        if self.frame >= 99:
            self.alpha = 255
            self.hit = False
            self.update = self.main_update
        elif self.frame >= 40:
            self.alpha = 255 - 90 + 90 * cos(2*2*pi*(self.frame - 40)/40.)
            self.scale_factor = 1.
            if self.frame == 40:
                self.x, self.y = 200.0, 100.0
            self.main_update(keystate)
        elif self.frame >= 20:
            self.alpha = 0
            self.frame += 1
        else:
            self.scale_factor += .1
            self.alpha -= 255 // 20
            self.frame += 1


    def update2(self, keystate):
        self.last_shot = -10
        self.last_bomb = -120
        self.alpha = 255

        if keystate & 32 and self.bombs > 0:
            self.launch_bomb()
            self.update = self.main_update
            self.frame += 1
            self.hit = False
            return
        if self.frame >= 6:
            self.lives -= 1
            self.frame = 0
            self.bombs = 1

            self.bomb_particles = []
            self.effects = [Sprite((32, 256-64, 64, 64), blend_mode=1, pos=(self.x, self.y))]
            self.effects[0].scale_factor = .2
            self.effects[0].alpha = 100

            self.update = self.update_dead
        self.frame += 1


    def main_update(self, keystate):
        # 1 = Up
        # 2 = Down
        # 4 = Left
        # 8 = Right
        # 16 = Fire
        # 32 = Secondary fire
        # 64 = Slow

        self.next_bomb_completion = (1 + ((self.frame - 1) % (60 * 40))) / (60. * 40)

        # Give the player one bomb every 40 seconds
        if self.next_bomb_completion == 1.:
            self.bombs += 1

        self.slow_mode = keystate & 64
        try:
            dx, dy = {1: (0, 1.), 2: (0, -1.), 4: (-1., 0), 8: (1., 0),
                      5: (-S2, S2), 9: (S2, S2),
                      6: (-S2, -S2), 10: (S2, -S2)}[keystate & 0xf]
        except KeyError:
            dx, dy = 0, 0
        else:
            speed = 2. if keystate & 64 else 4.8
            dx, dy = dx * speed, dy * speed

        self.x += dx
        self.y += dy

        (xmin, xmax, a, b), (c, ymin, ymax, d) = self.vertmat
        if self.x + xmin < 0:
            self.x = - xmin
        if self.x + xmax > self.game.width:
            self.x = self.game.width - xmax
        if self.y + ymin < 0:
            self.y = - ymin
        if self.y + ymax > self.game.height:
            self.y = self.game.height - ymax

        if keystate & 16 and self.last_shot < self.frame - 10:
            self.last_shot = self.frame
            bullet = Bullet((self.x, self.y + 20.), self.game.bullet_types[2][0],
                            pi / 2., 5.)
            self.game.player_bullets.append(bullet)
        if keystate & 32 and self.last_bomb < self.frame - 80 and self.bombs > 0:
            self.launch_bomb()

        if self.last_bomb > self.frame - 120:
            progress = (self.frame - self.last_bomb) / 120.
            self.alpha = 255 - 90 + 90 * cos(5*2*pi*progress)
            distance = 100. * sin(pi*progress)
            distance2 = (distance + 10.) ** 2
            angle = 8*pi*progress
            for i, particle in enumerate(self.bomb_particles):
                particle.x = self.x + cos(angle + 2*pi*i/7.) * distance
                particle.y = self.y + sin(angle + 2*pi*i/7.) * distance
            for bullet in self.game.bullets:
                d = (bullet.x - self.x) ** 2 + (bullet.y - self.y) ** 2
                if d <= distance2:
                    self.game.score += 5
                    bullet.set_disappearing(10, 0.5)
                    self.game.bullets.remove(bullet)
                    self.game.pending_deletion.append(bullet)
        elif self.last_bomb == self.frame - 120:
            self.alpha = 255
            self.bomb_particles = []

        self.frame += 1
