from math import cos, sin

class Sprite(object):
    def __init__(self, texcoords, automatic_rotation=False, blend_mode=0, angle=0., hitbox=None, pos=(0., 0.), offset=None):
        self.x, self.y = pos

        tx, ty, tw, th = texcoords
        self.center_offset = offset
        self.automatic_rotation = automatic_rotation
        self.hitbox = hitbox or (-tw / 3., -th / 3., tw / 3., th / 3.)
        self.vertmat = [[-tw/2.,  tw/2.,  tw/2., -tw/2.],
                        [-th/2., -th/2.,  th/2.,  th/2.]]

        self.blend_mode = blend_mode
        self.angle = angle
        self.scale_factor = 1.
        self.alpha = 255

        self.texcoords = (tx / 256.,            (256. - ty - th) / 256.,
                          (tx + tw) / 256.,     (256. - ty - th) / 256.,
                          (tx + tw) / 256.,     (256. - ty) / 256.,
                          tx / 256.,            (256. - ty) / 256.)


    def flip(self):
        # Not collision safe, but well
        (x1, x2, x3, x4) = self.vertmat[0]
        self.vertmat[0] = [x3, x4, x1, x2]


    def collide(self, other):
        thax, thay, thax2, thay2 = self.hitbox
        thbx, thby, thbx2, thby2 = other.hitbox
        thax += self.x ; thax2 += self.x
        thay += self.y ; thay2 += self.y
        thbx += other.x ; thbx2 += other.x
        thby += other.y ; thby2 += other.y
        if thax2 < thbx or thax > thbx2:
            return False
        if thay2 < thby or thay > thby2:
            return False
        return True


    def is_visible(self, width, height):
        (xmin, xmax, a, b), (ymin, c, ymax, d) = self.vertmat
        if xmax + self.x < 0 or xmin + self.x > width or ymax + self.y < 0 or ymin + self.y > height:
            return False
        return True

