import pyglet
from pyglet.gl import *

class MainWindow(pyglet.window.Window):
    def __init__(self, width, height):
        pyglet.window.Window.__init__(self, width, height, resizable=False)
        self.keys = pyglet.window.key.KeyStateHandler()
        self.push_handlers(self.keys)
        self.screens = []


    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.S:
            glViewport(0, 0, self.width, self.height)
            pyglet.image.get_buffer_manager().get_color_buffer().save('screen.png')

        if self.screens:
            self.screens[-1].on_key_press(symbol, modifiers)

        pyglet.window.Window.on_key_press(self, symbol, modifiers)


    def on_text(self, text):
        try:
            callback = self.screens[-1].on_text
        except (AttributeError, IndexError):
            pass
        else:
            callback(text)


    def on_text_motion(self, motion, select=False):
        try:
            callback = self.screens[-1].on_text_motion
        except (AttributeError, IndexError):
            pass
        else:
            callback(motion, select=False)


    def run(self):
        pyglet.clock.set_fps_limit(60)
        while self.screens:
            pyglet.clock.tick()
            self.dispatch_events()
            self.update()
            self.render()
            self.flip()


    def update(self):
        keystate = 0
        if self.keys[pyglet.window.key.UP]:
            keystate |= 1
        if self.keys[pyglet.window.key.DOWN]:
            keystate |= 2
        if self.keys[pyglet.window.key.LEFT]:
            keystate |= 4
        if self.keys[pyglet.window.key.RIGHT]:
            keystate |= 8
        if self.keys[pyglet.window.key.RIGHT]:
            keystate |= 8
        if self.keys[pyglet.window.key.LCTRL]:
            keystate |= 16
        if self.keys[pyglet.window.key.X]:
            keystate |= 32
        if self.keys[pyglet.window.key.LSHIFT]:
            keystate |= 64

        if self.screens:
            self.screens[-1].update(keystate)


    def render(self):
        if self.screens:
            self.screens[-1].render()

