import os
import time
import glob

from fyui.screens.common import MenuScreen, MenuItem
from fyui.screens.gamescreen import GameScreen
from fyui.game.replay import Replay
from fyui.game.gamestate import GameState
from fyui.game.game import Game

import pyglet


class ReplayGameScreen(GameScreen):
    def __init__(self, replay, screen_list):
        game = Game(400, 500, seed=replay.seed, seed2=replay.seed2,
                              survival_mode=replay.survival_mode)
        GameScreen.__init__(self, game, screen_list)
        self.replay = replay


    def update(self, keystate):
        if self.game.state != GameState.PLAYING:
            self.screen_list.remove(self)
            return
        if not self.paused:
            keystate = self.replay.get_keystate(self.frame)
            self.game.update(keystate)
            self.frame += 1



class ReplayScreen(MenuScreen):
    def __init__(self, screen_list):
        MenuScreen.__init__(self, screen_list)
        self.background = pyglet.resource.image('replay_screen.png')

        self.selected_button = 0

        common_params = {'color': (255, 255, 0, 255), 'bold': True, 'font_size': 18, 'anchor_y': 'top', 'y': 340}

        self.header = [pyglet.text.Label('Date', x=60, anchor_x='left', **common_params),
                       pyglet.text.Label('Seed', x=220, anchor_x='left', **common_params),
                       pyglet.text.Label('Score', x=440, anchor_x='right', **common_params)]

        common_params['bold'] = False
        common_params['font_size'] = 14
        y = common_params['y']
        del common_params['y']
        self.buttons = []
        self.seed_labels = []
        self.score_labels = []

        replay_glob = os.path.join(pyglet.resource.get_script_home(), 'replays', '*.rpy')
        replay_files = [(os.stat(path).st_ctime, path) for path in glob.glob(replay_glob)]
        replay_files.sort(reverse=True)

        for ctime, path in replay_files:
            ctime = time.gmtime(ctime)
            with open(path, 'rb') as file:
                replay = Replay.load(file)
            y -= 30
            self.buttons.append(MenuItem(self.make_callback(replay), time.strftime('%Y/%m/%d', ctime), x=60, anchor_x='left', y=y, **common_params))
            self.seed_labels.append(pyglet.text.Label('%d' % replay.seed, x=220, anchor_x='left', y=y, **common_params))
            self.score_labels.append(pyglet.text.Label('%010d' % replay.score, x=440, anchor_x='right', y=y, **common_params))


    def make_callback(self, replay):
        def callback():
            replay_game_screen = ReplayGameScreen(replay, self.screen_list)
            self.screen_list.append(replay_game_screen)
        return callback


    def update(self, keystate):
        pass


    def render(self):
        self.background.blit(0, 0)
        for label in self.header:
            label.draw()
        for i, (button, seed_label, score_label) in enumerate(zip(self.buttons, self.seed_labels, self.score_labels)):
            if self.selected_button == i:
                button.color = (255, 255, 0, 255)
                seed_label.color = (255, 255, 0, 255)
                score_label.color = (255, 255, 0, 255)
            else:
                button.color = (255, 255, 255, 255)
                seed_label.color = (255, 255, 255, 255)
                score_label.color = (255, 255, 255, 255)
            button.draw()
            seed_label.draw()
            score_label.draw()

