import os
from operator import itemgetter

import json
import pyglet



class ScoresScreen(object):
    def __init__(self, screen_list, game=None, seed=0):
        self.screen_list = screen_list

        self.background = pyglet.resource.image('scores_screen.png')
        common_params = {'color': (255, 255, 0, 255), 'bold': True, 'font_size': 18, 'anchor_y': 'top', 'y': 380}

        self.seed = str(game.seed if game else seed)

        self.seed_label = pyglet.text.Label('Seed: %s' % self.seed, x=20, **common_params)
        self.seed_label.y = 420
        self.header = [pyglet.text.Label('#', x=20, anchor_x='left', **common_params),
                       pyglet.text.Label('Name', x=60, anchor_x='left', **common_params),
                       pyglet.text.Label('Score', x=480, anchor_x='right', **common_params)]

        self.document = pyglet.text.document.UnformattedDocument('')
        self.layout = pyglet.text.layout.IncrementalTextLayout(self.document, 160, 30, multiline=False)
        self.caret = pyglet.text.caret.Caret(self.layout)
        self.on_text = self.caret.on_text
        self.on_text_motion = self.caret.on_text_motion
        self.caret.visible = True

        try:
            with open(os.path.join(pyglet.resource.get_script_home(), 'scores.dat'), 'r') as file:
                self.scores = json.load(file)
        except IOError:
            self.scores = {}

        if not self.seed in self.scores:
            self.scores[self.seed] = []

        if game:
            self.scores[self.seed].append((None, game.score))
        self.scores[self.seed].sort(key=itemgetter(-1), reverse=True)
        self.scores[self.seed] = self.scores[self.seed][:5]

        self.score_labels = []
        for rank, (name, score) in enumerate(self.scores[self.seed]):
            rank = rank + 1
            common_params['y'] -= 30
            if name is not None:
                common_params['color'] = (255, 255, 210, 255)
            else:
                common_params['color'] = (255, 255, 127, 255)
            common_params['bold'] = False
            labels = []
            labels.append(pyglet.text.Label('%d' % rank, x=20, anchor_x='left', **common_params))
            if name is not None:
                labels.append(pyglet.text.Label(name, x=60, anchor_x='left', **common_params))
            else:
                self.document.set_style(0, 1, common_params)
                labels.append(self.layout)
                self.caret.set_style(common_params)
                self.layout.x = 60
                self.layout.anchor_y = 'top'
                self.layout.y = common_params['y']
            labels.append(pyglet.text.Label('%010d' % score, x=480, anchor_x='right', **common_params))
            self.score_labels.append(labels)


    def save_scores(self):
        self.scores[self.seed] = []
        for _, name_label, score_label in self.score_labels:
            score = long(score_label.text)
            if hasattr(name_label, 'text'):
                name = name_label.text
            else:
                name = self.document.text
            self.scores[self.seed].append((name, score))
        with open(os.path.join(pyglet.resource.get_script_home(), 'scores.dat'), 'w') as file:
            json.dump(self.scores, file)
        assert file.closed


    def on_key_press(self, key, modifiers):
        if key == pyglet.window.key.ENTER:
            self.save_scores()
            self.screen_list.remove(self)
        elif key in (pyglet.window.key.ESCAPE,):
            self.screen_list.remove(self)


    def update(self, keystate):
        pass


    def render(self):
        self.background.blit(0, 0)
        self.seed_label.draw()
        for i in self.header:
           i.draw()
        for labels in self.score_labels:
            for label in labels:
                label.draw()

