import os

import pyglet

from fyui.screens.common import MenuScreen, MenuItem
from fyui.screens.scoresscreen2 import ScoresScreen2
from fyui.screens.campaignscreen import CampaignScreen
from fyui.screens.survivalscreen import SurvivalScreen
from fyui.screens.replayscreen import ReplayScreen


class TitleScreen(MenuScreen):
    def __init__(self, screen_list):
        MenuScreen.__init__(self, screen_list)
        self.background = pyglet.resource.image('title_screen.png')

        self.selected_button = 0

        common_params = {'bold': True, 'font_name': 'Sans', 'font_size': 30, 'x': 510,
                         'anchor_x': 'right', 'anchor_y': 'center'}
        self.buttons = [MenuItem(self.start_campaign, 'Campaign', y=290, **common_params),
                        MenuItem(self.start_survival, 'Survival', y=240, **common_params),
                        MenuItem(self.start_replays, 'Replays', y=190, **common_params),
#                        MenuItem(lambda: None, 'Awards', y=140, **common_params),
                        MenuItem(self.start_scores, 'Scores', y=90+50, **common_params),
                        MenuItem(self.quit, 'Quit', y=40+50, **common_params)]


    def start_replays(self):
        replays_screen = ReplayScreen(self.screen_list)
        self.screen_list.append(replays_screen)


    def start_scores(self):
        scores_screen = ScoresScreen2(self.screen_list)
        self.screen_list.append(scores_screen)


    def start_survival(self):
        survival_screen = SurvivalScreen(self.screen_list)
        self.screen_list.append(survival_screen)


    def start_campaign(self):
        campaign_screen = CampaignScreen(self.screen_list)
        self.screen_list.append(campaign_screen)


    def update(self, keystate):
        pass


    def render(self):
        self.background.blit(0, 0)
        for i, button in enumerate(self.buttons):
            if self.selected_button == i:
                button.color = (255, 255, 0, 255)
            else:
                button.color = (255, 255, 255, 255)
            button.draw()

