from fyui.utils.value_translation import *

class MetaRegistry(type):
    def __new__(mcs, name, bases, dict):
        dict['_handlers'] = {}
        for key, value in dict.items():
            try:
                for i in value._handled_opcodes:
                    dict['_handlers'][i] = value
            except AttributeError:
                pass
        return type.__new__(mcs, name, bases, dict)


def register(opcodes):
    def _decorator(func):
        func._handled_opcodes = opcodes
        return func
    return _decorator



class ParseStop(Exception):
    pass #TODO



class BossRunner(object):
    __metaclass__ = MetaRegistry


    def __init__(self, boss):
        self.boss = boss
        self.selected_script = None


    def reload_script(self):
        self.boss.speed_modifier = 1.
        self.boss.autofire = (0, 0, 2, 0.4)
        self.boss.life = 0
        self.boss.rotation_speed = 0.01
        self.boss.exec_frame = 0
        self.boss.exec_pointer = 0
        self.boss.script = []
        self.selected_script = self.boss.script
        try:
            for opcode, p1, p2, p3, p4 in self.boss.data:
                self._handlers.get(opcode, lambda x, opcode, p1, p2, p3, p4: None)(self, opcode, p1, p2, p3, p4)
        except ParseStop:
            return True
        return False


    def _get_script(self, delay):
        if delay == 0 and self.selected_script:
            script = self.selected_script[-1][1]
        else:
            script = []
            last_frame = self.selected_script[-1][0] if self.selected_script else 0
            self.selected_script.append((last_frame + delay, script))
        return script


    @register((0,))
    def post(self, opcode, p1, p2, p3, p4):
        if p1 or p2 or p3 or p4:
            return
        else:
            raise ParseStop


    @register((1, 2))
    def set_life(self, opcode, life, *args):
        self.boss.life += ubyte_to_value[life] // 4


    @register((4,))
    def set_boss_wings(self, opcode, p1, p2, p3, p4):
        wings_texcoords = [(85, 0, 72, 45), (157, 0, 72, 45), (85, 45, 72, 45)]
        self.boss.set_wings(wings_texcoords[(ubyte_to_value[p1 | p2 | p3 | p4] >> 3) % 3])


    @register((8, 16, 32, 64, 128, 3, 5, 6, 9, 144, 160, 192))
    def parse_set_bullet_type(self, opcode, delay, types, number, extra_param):
        delay, types, number = (ubyte_to_value[i] for i in (delay, types, number))
        script = self._get_script(delay)
        set_type = types & 3
        bullet_type = types >> 2
        script.append((1, (set_type, bullet_type, number, r8_to_value[extra_param])))


    @register((10, 12, 17, 18, 20, 24, 33, 34, 36, 40, 48, 65, 66, 68, 72, 80, 96, 129, 130, 132, 136,
               7, 11, 13, 14, 19, 21, 22, 25, 26, 28, 35))
    def parse_fire(self, opcode, delay, types, number, extra_param):
        delay, types, number = (ubyte_to_value[i] for i in (delay, types, number))
        script = self._get_script(delay)
        set_type = types & 3
        bullet_type = types >> 2
        script.append((2, (set_type, bullet_type, number, r8_to_value[extra_param])))


    @register((92, 99, 101, 102, 105, 106, 108, 113, 114, 116, 120, 135, 139, 141, 142, 147, 149, 150, 153, 154, 156, 163, 165, 166, 169, 170, 172, 177, 178, 180, 184, 195, 197, 198, 201, 202, 204, 209, 210, 212, 216,))
    def parse_fire_nodelay(self, opcode, types, number, extra_param, *args):
        types, number = (ubyte_to_value[i] for i in (types, number))
        script = self._get_script(0)
        set_type = types & 3
        bullet_type = types >> 2
        script.append((2, (set_type, bullet_type, number, r8_to_value[extra_param])))


    @register((37, 38, 41, 42, 44, 49, 50, 52,
               15, 23, 27, 29, 30, 39, 43, 45, 46, 51, 53, 54, 57, 58, 60, 71, 75, 77, 78, 83, 85, 86, 89, 90))
    def parse_speed_modifier_change(self, opcode, speed_modifier, *args):
        script = self._get_script(0)
        script.append((4, (r8_to_value[speed_modifier],)))


    @register((31, 47, 55, 59, 61, 62, 79, 87, 91, 93, 94, 103, 107, 109, 110, 115, 117, 118, 121, 122, 124, 143, 151, 155, 157, 158, 167, 171, 173, 174, 179, 181, 182, 185, 186, 188,
               63, 95, 111)) #TODO
    def parse_change_properties(self, opcode, bullet_type, accel, rotation_speed, extra):
        script = self._get_script(0)
        bullet_type = ubyte_to_value[bullet_type]
        script.append((3, (bullet_type, r8_to_value[accel], r8_to_value[rotation_speed])))


    @register((127, 191, 223, 229, 247, 251, 253, 254))
    def parse_drone_func(self, opcode, delay, types, number, extra_param):
        types, number = (ubyte_to_value[i] for i in (types, number))
        script = self._get_script(0)
        set_type = types & 3
        types >>= 2
        drone_id = types & 1
        bullet_type = types >> 1
        script.append((5, (drone_id, set_type, bullet_type, number, r8_to_value[extra_param])))

